<?php

/**
 * @file
 * Contains \Drupal\bootstrap_colors\Form\ColorForm.
 */

namespace Drupal\bootstrap_colors\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

class ColorForm extends FormBase {
  
  /**
   * {@inheritdoc}.
   */
  public function getFormId() {
    return 'bootstrap_colors_color_form';
  }
  
  /**
   * {@inheritdoc}.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attributes']['class'][] = 'clearfix';
    $form['primary_shade'] = array(
     '#type' => 'textfield',
     '#title' => t('Primary shade'),
     '#size' => 20,
     '#maxlength' => 7,
     '#default_value' => theme_get_setting('bootstrap_barrio_base_primary_color') ?? '#000000',
     '#attributes' => array('id' => array('primary-shade'), 'class' => array('white')),
     '#required' => TRUE,
    );
    $form['primary_light'] = array(
     '#type' => 'textfield',
     '#title' => t('Primary light'),
     '#size' => 20,
     '#maxlength' => 7,
     '#attributes' => array('id' => array('primary-light'), 'class' => array()),
     '#disabled' => TRUE,
    );
    $form['primary_dark'] = array(
     '#type' => 'textfield',
     '#title' => t('Primary dark'),
     '#size' => 20,
     '#maxlength' => 7,
     '#attributes' => array('id' => array('primary-dark'), 'class' => array('white')),
     '#disabled' => TRUE,
    );
    $form['accent_shade'] = array(
     '#type' => 'textfield',
     '#title' => t('Accent shade'),
     '#size' => 20,
     '#maxlength' => 7,
     '#default_value' => theme_get_setting('bootstrap_barrio_base_secondary_color') ?? '#000000',
     '#attributes' => array('id' => array('accent-shade'), 'class' => array('white')),
     '#required' => TRUE,
    );
    $form['accent_light'] = array(
     '#type' => 'textfield',
     '#title' => t('Accent light'),
     '#size' => 20,
     '#maxlength' => 7,
     '#attributes' => array('id' => array('accent-light'), 'class' => array()),
     '#disabled' => TRUE,
    );
    $form['accent_dark'] = array(
     '#type' => 'textfield',
     '#title' => t('Accent dark'),
     '#size' => 20,
     '#maxlength' => 7,
     '#attributes' => array('id' => array('accent-dark'), 'class' => array('white')),
     '#disabled' => TRUE,
    );
    $form['body_bg'] = array(
      '#type' => 'textfield',
      '#title' => t('Body Background'),
      '#size' => 20,
      '#maxlength' => 7,
      '#default_value' => theme_get_setting('bootstrap_barrio_body_bg_color') ?? '#ffffff',
      '#attributes' => array('id' => array('body_bg'), 'class' => array()),
      '#required' => TRUE,
      '#prefix' => '<div id="body_bg-wrap" class="colorpicker-element input-group">',
      '#suffix' => '<span class="add-on"><i id="body_bg-output"></i></span></div>'
    );
    $form['body'] = array(
      '#type' => 'textfield',
      '#title' => t('Text color'),
      '#size' => 20,
      '#maxlength' => 7,
      '#default_value' => theme_get_setting('bootstrap_barrio_body_color') ?? '#343a40',
      '#attributes' => array('id' => array('body'), 'class' => array()),
      '#required' => TRUE,
      '#prefix' => '<div id="body-wrap" class="colorpicker-element input-group">',
      '#suffix' => '<span class="add-on"><i id="body-output"></i></span></div>'
    );
    $form['h1'] = array(
      '#type' => 'textfield',
      '#title' => t('H1'),
      '#size' => 20,
      '#maxlength' => 7,
      '#default_value' => theme_get_setting('bootstrap_barrio_h1_color') ?? '#343a40',
      '#attributes' => array('id' => array('h1'), 'class' => array()),
      '#required' => TRUE,
      '#prefix' => '<div id="h1-wrap" class="colorpicker-element input-group">',
      '#suffix' => '<span class="add-on"><i id="h1-output"></i></span></div>'
    );
    $form['h2'] = array(
      '#type' => 'textfield',
      '#title' => t('H2'),
      '#size' => 20,
      '#maxlength' => 7,
      '#default_value' => theme_get_setting('bootstrap_barrio_h2_color') ?? '#343a40',
      '#attributes' => array('id' => array('h2'), 'class' => array()),
      '#required' => TRUE,
      '#prefix' => '<div id="h2-wrap" class="colorpicker-element input-group">',
      '#suffix' => '<span class="add-on"><i id="h2-output"></i></span></div>'
    );
    $form['h3'] = array(
      '#type' => 'textfield',
      '#title' => t('H3'),
      '#size' => 20,
      '#maxlength' => 7,
      '#default_value' => theme_get_setting('bootstrap_h3_color') ?? '#343a40',
      '#attributes' => array('id' => array('h3'), 'class' => array()),
      '#required' => TRUE,
      '#prefix' => '<div id="h3-wrap" class="colorpicker-element input-group">',
      '#suffix' => '<span class="add-on"><i id="h3-output"></i></span></div>'
    );
    $form['actions']['#type'] = 'actions';

    $form['actions']['save'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    );
    $form['actions']['cancel'] = array(
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#attributes' => array('class' => array('btn-secundary', 'btn')),
      '#submit' => array('_bootstrap_colors_cancel_submit'),
    );
    
    $form['#attached']['drupalSettings']['bootstrap_colors'] = [
      'primary_shade' => theme_get_setting('bootstrap_barrio_base_primary_color'),
      'accent_shade' => theme_get_setting('bootstrap_barrio_base_secondary_color'),
    ];

    return $form;
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $primary_shade = $form_state->getValue('primary_shade');
    $primary_light = $form_state->getValue('primary_light');
    $primary_dark = $form_state->getValue('primary_dark');
    $accent_shade = $form_state->getValue('accent_shade');
    $accent_light = $form_state->getValue('accent_light');
    $accent_dark = $form_state->getValue('accent_dark');
    $body_bg = $form_state->getValue('body_bg');
    $body = $form_state->getValue('body');
    $h1 = $form_state->getValue('h1');
    $h2 = $form_state->getValue('h2');
    $h3 = $form_state->getValue('h3');

    $theme_name = \Drupal::service('theme.manager')->getActiveTheme()->getName();
    $config = \Drupal::service('config.factory')->getEditable($theme_name . '.settings');

    $config->set('bootstrap_barrio_enable_color', TRUE)->save();
    $config->set('bootstrap_barrio_base_primary_color', $accent_shade)->save();
    $config->set('bootstrap_barrio_base_secondary_color', $primary_shade)->save();
    $config->set('bootstrap_barrio_body_color', $body)->save();
    $config->set('bootstrap_barrio_body_bg_color', $body_bg)->save();
    $config->set('bootstrap_barrio_h1_color', $h1)->save();
    $config->set('bootstrap_barrio_h2_color', $h2)->save();
    $config->set('bootstrap_barrio_h3_color', $h3)->save();
  }

  private function _bootstrap_colors_cancel_submit() {
    location.reload();
  }

}
